package de.fhdw.gaming.ipspiel23.memory;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * 
 * Implements {@link IGameMemoryProvider}.
 */
public final class GameMemoryProvider implements IGameMemoryProvider {
    
    /**
     * instance.
     */
    private static final GameMemoryProvider INSTANCE = new GameMemoryProvider();
    /**
     * Storage for the different memories.
     */
    private final Map<IGameMemoryIdentifier, IGameMemory<?>> memoryStore = new HashMap<>();

    /**
     * private Constructor.
     */
    private GameMemoryProvider() {
        
    }

    /**
     * Returns this memory provider.
     * @return this.
     */
    public static IGameMemoryProvider getInstance() {
        return INSTANCE;
    }
    
    
    @SuppressWarnings("unchecked")
    @Override
    public <ROUND_DATA> IGameMemory<ROUND_DATA> requestMemoryForStrategy(final IGameMemoryIdentifier reuseIdentifier,
            final IGameMemoryCapacity capacity) {
        final IGameMemory<?> memoryObj = this.memoryStore.getOrDefault(reuseIdentifier, null);
        if (memoryObj == null) {
            final IGameMemory<ROUND_DATA> memory = new GameMemory<>(capacity);
            this.memoryStore.put(reuseIdentifier, memory);
            return memory;
        }
        return (IGameMemory<ROUND_DATA>) memoryObj;
    }

    @SuppressWarnings("unchecked")
    @Override
    public <ROUND_DATA> Optional<IGameMemory<ROUND_DATA>> tryRequestMemoryForStrategy(
            final IGameMemoryIdentifier reuseIdentifier) {
        final IGameMemory<?> memoryObj = this.memoryStore.getOrDefault(reuseIdentifier, null);
        if (memoryObj == null) {
            return Optional.empty();
        }
        return Optional.of((IGameMemory<ROUND_DATA>) memoryObj);
    }
}
